/* all_bz -- generate all blastz commands for pairs of specified sequences.
*
*  The command syntax is:
*	all_bz [-+]string-with-names1 string-with-names2 ...
*  where each argument is a string containng names of sequence file (and
*  perhaps non-name characters.  Examples include:
*	all_bz human mouse rat
*	all_bz "(((human chimp) baboon) (mouse rat))"
*  The command
*	all_bz + human mouse rat
*  echos each blastz command to stdout before executing it.  The command
*  	all_bz - human mouse rat
*  simply echos the commands (which can be captured in a file, edited, then
*  executed).
*/

#define NON_NAME " ()"
#define BZ_CMD "blastz %s %s B=0 C=2 Y=3400 H=2000 | strict_chain > %s.%s.str"

#include "lib.h"

struct name_record {
	char *name;
	struct name_record *next;
};

int main(int argc, char **argv) {
	struct name_record *m, *n, *names;
	char *s, *t, x, cmd[10000];
	int i, execute = 1, verbose = 0;

	argv0 = "all_bz";
	if (argc < 2)
		fatal("args = [-+] string-with-names [string-with-names ..]");
	if (same_string(argv[1], "-")) {
		execute = 0;
		verbose = 1;
		--argc;
		++argv;
	} else if (same_string(argv[1], "+")) {
		verbose = 1;
		--argc;
		++argv;
	}
	
	m = names = NULL;
	for (i = 1; i < argc; ++i)
		for (s = argv[i]; *s != '\0'; s = t) {
			while (*s != '\0' && strchr(NON_NAME,*s))
				++s;
			if (*s == '\0')
				break;
			for (t = s+1; *t != '\0' && !strchr(NON_NAME,*t); ++t)
				;
			x = *t;	// save last character
			*t = '\0';
			n = ckalloc(sizeof(*n));
			n->name = copy_string(s);
			*t = x;	// replace last character
			n->next = NULL;
			if (m == NULL)
				names = n;
			else
				m->next = n;
			m = n;
		}
	for (m = names; m != NULL; m = m->next)
		for (n = m->next; n != NULL; n = n->next) {
			sprintf(cmd, BZ_CMD, m->name, n->name, m->name,
				n->name);
			if (verbose)
				printf("%s\n", cmd);
			if (execute && system(cmd) != 0)
				fatal("tba quits");;
		}
	return 0;
}
