// depth_maf.c -- determine frequencies of various depths of columns
// This version is for projections (so number of unaligned postions makes sense)

#include "lib.h"
#include "maf.h"
#include "mz_util.h"

#define MAX_SEQ_LEN 10000000

int deep[MAX_SEQ_LEN+1];

int main(int argc, char **argv) {
	struct mafFile *mf;
	struct mafAli *a;
	struct mafComp *c;
	char *src = NULL;
	int max_pos = 0, i, d, nrow, end, max_depth, *depth, nalign; 

	if (argc != 1 && argc != 2)
		fatal("optional arg = maf-file");
	if (argc == 1)
		mf = mafOpen("/dev/stdin", 1);
	else
		mf = mafOpen(argv[1], 1);
	max_depth = nalign = 0;
	while ((a = mafNext(mf)) != NULL) {
		++nalign;
		c = a->components;
		if (src == NULL)
			src = copy_string(c->src);
		else
			if (!same_string(src, c->src))
				fatalf("blocks differ in top row: %s and %s",
				  src, c->src);
		for (nrow = 0; c != 0; c = c->next, ++nrow)
			;
		if (nrow == 0) {
			fprintf(stderr, "alignment with 0 rows:\n");
			mafWrite(stderr, a);
			exit(1);
		}

		max_depth = MAX(nrow, max_depth);
		c = a->components;
		end = c->start + c->size - 1;
		if (end > MAX_SEQ_LEN)
			fatalf("end > %d", MAX_SEQ_LEN);
		max_pos = MAX(max_pos, end);
		for (i = c->start; i <= end; ++i)
			deep[i] = MAX(deep[i],nrow);
	}
	if (nalign == 0)
		fatal("maf_depth: no alignments were given");
	//printf("  %d alignments, max_pos = %d\n", nalign, max_pos);
	printf("  %d alignments\n", nalign);
	depth = ckalloc((max_depth+1)*sizeof(int));
	for (i = 0; i <= max_depth; ++i)
		depth[i] = 0;
	for (i = 0; i <= max_pos; ++i) {
		if ((d = deep[i]) > max_depth || d < 1)
			fatalf("deep[%d] = %d\n", i, deep[i]);
		++depth[d];
	}
	for (i = 1; i <= max_depth; ++i)
		printf("%4d: %d\n", i, depth[i]);
	return 0;
}
