// maf_split -- split an alignment into two sets of rows

#include "util.h"
#include "maf_split.h"
#include "maf.h"
#include "mz_scores.h"

// #define VERBOSE

// remove columns containing only dashes
static void dash_cols(struct mafAli *a) {
	int i, j;
	struct mafComp *c;

	for (i = j = 0; i < a->textSize; ++i) {
		for (c = a->components; c != NULL; c = c->next)
			if (c->text[i] != '-')
				break;
		if (c != NULL) {
			if (j != i)
				for (c = a->components; c != NULL; c = c->next)
					c->text[j] = c->text[i];
			++j;
		}
	}
	if (j != a->textSize) {
		a->textSize = j;
		for (c = a->components; c != NULL; c = c->next)
			c->text[j] = '\0';
	}
}

// Entry point: (documentation in maf_split.h)
struct mafAli *split(struct mafAli *a, char **source, int ntop, int nseq) {
	int i;
	struct mafAli *b = ckalloc(sizeof(*b));
	struct mafComp *c, *d, *enda, *endb;

#ifdef VERBOSE
	fprintf(stderr, "%d columns in a deconstructed block:\n",
	  a->textSize);
	//mafWrite(stderr, a);
#endif

	b->textSize = a->textSize;
	c = a->components;
	a->components = b->components = enda = endb = NULL;

	for ( ; c != NULL; c = d) {
		d = c->next;
		c->next = NULL;
		for (i = 0; i < nseq; ++i)
			if (same_string(c->src, source[i]))
				break;
		if (i == nseq)
			fatalf("did not find source %s", c->src);
		if (i < ntop) {
			if (enda == NULL)
				a->components = c;
			else
				enda->next = c;
			enda = c;
		} else {
			if (endb == NULL)
				b->components = c;
			else
				endb->next = c;
			endb = c;
		}
	}
	dash_cols(a);
	dash_cols(b);
	a->score = mafScoreRange(a, 0, a->textSize);
	b->score = mafScoreRange(b, 0, b->textSize);
#ifdef VERBOSE
	fprintf(stderr, "resulting blocks are:\n");
	mafWrite(stderr, a);
	mafWrite(stderr, b);
#endif
	return b;
}
